/*
 Low Voltage Battery Monitor
 Version: 1.1
 Author: Alex from insideGadgets (http://www.insidegadgets.com)
 Created: 4/12/2011
 Last Modified: 21/01/2012
 
 A 4x AA battery monitor which will blink an LED every 4 seconds if a battery
 is found flat (1.1V) and checks good batteries every 15 minutes. Suitable 
 for rechargeable batteries.
 
 */
 
// ATtiny24/44/84 Pin map
//
//               +-\/-+
//         VCC  1|o   |14  GND
//         PB0  2|    |13  PA0 (ADC0)
//         PB1  3|    |12  PA1 (ADC1)
//   Reset PB3  4|    |11  PA2 (ADC2)
//         PB2  5|    |10  PA3 (ADC3)
//  (ADC7) PA7  6|    |9   PA4 (ADC4)
//  (ADC6) PA6  7|    |8   PA5 (ADC5)
//               +----+
//

#define F_CPU 1000000 // 1 MHz clock

#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/sleep.h>
#include "setup.c"

int voltagePin = 7; // (ADC7) PA7
#define batt1 (1<<PA0)
#define batt2 (1<<PA1)
#define batt3 (1<<PA2)
#define batt4 (1<<PA3)
#define led1 (1<<PA4)
#define led2 (1<<PA5)
#define led3 (1<<PA6)
#define led4 (1<<PB2)

#define COUNTSTART 226
#define TIMETOCHECK 1
#define LOWVOLT 0

// Battery count down counters, used so when know when to re-check voltage
byte batteries[4] = {1, 1, 1, 1};

int main(void) {
	
	setup();
	
	// Set Outputs
	DDRA |= (batt1 | batt2 | batt3 | batt4 | led1 | led2 | led3);
	DDRB |= (led4);
	
	// Turn on pull-up resistors to save power on un-used ports
	PORTB |= ((1<<PB0) | (1<<PB1));
	
	while(1) {
		
		int battNum = 0;
		for (battNum = 0; battNum < 4; battNum++) {
			if (batteries[battNum] <= TIMETOCHECK) { // Time to check battery
				PORTA |= (1<<battNum); // Turn on Mosfet for the corresponding battery (PA0-3)
				setup_watchdog(T32MS); // Wait for mosfet to turn on
				system_sleep();
				turnOffwatchdog();
				
				int voltageValue = analogRead(voltagePin);
				PORTA = 0x00; // Turn off Mosfet
				
				if (voltageValue > 0 && voltageValue < 512) { // If more than 0V but less than 1.1V
					batteries[battNum] = LOWVOLT; // Battery low, keep re-checking this battery every 4 seconds
					
					// Light LED for 0.5 seconds
					if (battNum < 3) {
						PORTA |= (1<<(battNum + 4));
					}
					else {
						PORTB |= led4;
					}
					
					setup_watchdog(T500MS);
					system_sleep();
					turnOffwatchdog();
					
					PORTA = 0x00; // LEDs off
					PORTB &= ~led4;
				}
				else if (voltageValue >= 512) {
					batteries[battNum] = COUNTSTART; // Battery ok, wait 5 minutes before re-checking this battery
				}
				else {
					batteries[battNum] = TIMETOCHECK; // No battery present, check again in 4 seconds
				}
			}
			else {
				batteries[battNum]--;
			}
		}
		
		setup_watchdog(T4S);
		system_sleep();
		turnOffwatchdog();
	}
}

ISR(WDT_vect) { }
